/**
 * @file  Channel1_2.h
 * @brief Schnittstelle fuer die Klasse CChannel1_2.
 */
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_AE3MODBUS_H__783E8448_0B59_4DCC_B429_BDCAA018A25B__INCLUDED_)
#define AFX_AE3MODBUS_H__783E8448_0B59_4DCC_B429_BDCAA018A25B__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "ChannelBase.h"
#include "../Common/AEErrorDef.h"
#include "../Common/aes.h"

// Datensatzbefehle und Antworten

// Datensatzanfrage
#define MB_CMD_READ_RECORD              0x40  // Datensatz senden

#define MB_ANSW_NO_RECORD               0xC0  // kein Datensatz vorhanden
#define MB_ANSW_OFFLINE_RECORD          0xC1  // Offline Datensatz
#define MB_ANSW_OLD_OFFLINE_RECORD      0xC3  // wiederholt gesendeter Offline Datensatz
#define MB_ANSW_ONLINE_RECORD           0xE1  // Online Datensatz
#define MB_ANSW_OLD_ONLINE_RECORD       0xE3  // wiederholt gesendeter Online Datensatz

// Datensatzquittierung
#define MB_CMD_ACK_RECORD               0x48  // Quittung fuer Datensatz

#define MB_ANSW_ACK_OFF_RECORD	        0xC8  // Quittung erhalten (fuer Offline-Datensatz)
#define MB_ANSW_OLD_ACK_OFF_RECORD      0xCA  // wiederholt gesendete Quittung
#define MB_ANSW_ACK_ON_RECORD           0xE8  // Quittung erhalten (fuer Online-Datensatz)
#define MB_ANSW_OLD_ACK_ON_RECORD       0xEA  // wiederholt gesendete Quittung
#define MB_ANSW_TIMEOUT_ACK_RECORD      0xE9  // Quittung kam nach Timeout (fuer Online-Datensatz)
#define MB_ANSW_OLD_TIMEOUT_ACK_RECORD  0xEB  // wiederholte Quittung nach Timeout

// Datensatzloeschbefehl
#define MB_CMD_DEL                      0x49  // Loescht alle Datensaetze.

// alte Befehle des AEIII+
#define CMD_READ        0x31
#define CMD_READ_AGAIN  0x32
#define CMD_DEL         0x33

// alte Antworten des AEIII+
#define ANSW_DAT_NOANSW 0x31 // Keine Antwort vom PC erwartet
#define ANSW_DAT_ANSW   0x71 // Antwort vom PC erwartet
#define ANSW_NODAT      0xB1 // Keine Datem im Geraet


#include "../Common/AEErrorDef.h"

// Adresse des Pointers der im EEPROM liegt und aud das Flash zeigt
#define P_LIST_STRUCT   0x0024 // Startadresse Listenbeschreibung
#define P_DAT_STRUCT    0x0080 // Kopie vonP_DAT_STRUCT Fehler von ATMega zu umgehen

// Datafox
#define MB_CONTROL_STOP     0x48000000
#define MB_RECORD_AVAILABLE 0x48000001

#define DISPLAY_MEMORY      0x49000000
#define INFOTEXT_MEMORY     0x4B000000

#define MB_SET_TIME         0x43000000 // Offset fuer Uhr schreiben
#define MB_GET_VERSION      0x42000000 // Offset fuer Version lesen
#define MB_GET_TIME         0x44000000 // Offset fuer Uhr lesen

#define MB_START_EEP        0x40000000 // Startadresse EEP-ROM
#define MB_SERIEN_MEMORY    0x4A000800

#define MB_START_CONFIG     0x4A000000 // Startadresse AE-Konfiguration
#define MB_START_IOMODULE   0x4C000000 // Startadresse IOModule - Struktur
#define MB_DIGITAL_IO       0x4D000000 // Startadresse DigitalMDE - Struktur fuer IOModul
#define MB_DIGITAL_MDE      0x4E000000 // Startadresse DigitalMDE - Struktur fuer Interne Digitaleingaenge
#define MB_MDE_EK           (FLASH_KONFIG_START + FLASH_SIZE_SU) // Startadresse Eingabeketten - Strukturen fuer MDE

// Tabellen fuer Zutrittskontrolle
#define MB_ENTRANCE_TABLE_0 0x4F000000
#define MB_ENTRANCE_TABLE_1 0x4F100000
#define MB_ENTRANCE_TABLE_2 0x4F200000
#define MB_ENTRANCE_TABLE_3 0x4F300000

// Eingabeketten fuer Zutrittskontrolle
#define MB_ENTRANCE_EK      (FLASH_KONFIG_START + FLASH_SIZE_SU + EXTENDED_SIZE_SU) // Startadresse Eingabeketten - Strukturen fuer Zutrittskontrolle

// Datenaufbau: 
// 1tes Byte:
// bit(7) gesetzt ist setzen. nicht gesetzt ist lesen
// bit(6) .. bit(0) ist Nummer.
// 2tes Byte: frei bei Relais und Laenge fuer GlobVars
// 3tes und 4tes Byte:
// Dauer fuer Relais, bei GlobVar muss nicht vorhanden sein.
#define MB_RELAY            0x50000000  // Startadresse Relais
#define MB_GLOBAL_VAR       0x51000000  // Startadresse Globale Variable

// 32 Byte Zeichenabfrage fuer Firmwareversion
#define MB_VERSION          0x5D000000 
// Starten der Datenuebertragung und des Bootloaderresets
#define MB_BOOTLOAD         0x5E000000 
#define MB_RESTART          0x5C000000 // Neustart PZE Master IV

// lesen/schreiben der Speicherbereiche
#define MB_ACCESS_SRAM       0xF0000000
#define MB_ACCESS_EEPROM     0xF1000000
#define MB_ACCESS_FLASH      0xF2000000
#define MB_ACCESS_PROGRAM    0xF3000000
#define MB_ACCESS_FLASH_PAGE 0xF4000000
#define MB_ACCESS_MMC        0xF5000000

#define MAX_MB_PAKET         0xFF // max. MOD-Bus Datenpaket
#define MAX_MB_PAKET_2       (MAX_MB_PAKET + 21)
#define MAX_MB_PAKET_HEADER  13
#define MAX_MB_PAKET_DATA    (MAX_MB_PAKET - MAX_MB_PAKET_HEADER)
#define MAX_MB_PAKET_SYSTEM_DATA 0x200

#define MAX_WBUF    MAX_MB_PAKET
#define MAX_RBUF    MAX_MB_PAKET

#define MB_READ     0x21 // bis zu 242 Byte Daten lesen
#define MB_WRITE    0x22 // bis zu 242 Byte Daten schreiben

#define MB_ADD_OFFLINE  0xFE

extern const int g_iErrorMap[];

#include "Channel2Def.h"

// Modbusprotokoll fuer AEIII+ verwalten.
class CChannel1_2
{
	friend class CAEChannel;
	friend class CAE3Firmware;
public:
	CChannel1_2() { InitClass(); }
	CChannel1_2(short channelID, short deviceID) 
	{ InitClass(channelID, deviceID); }
	virtual void InitClass(short channelID = 0, short deviceID = 31);
	virtual ~CChannel1_2();

public:
	void SetCBFunction(int (*pFunction)());
	int ExeCallBack();

	void UShortToBin(unsigned short Val,unsigned char *Buf);	
	void ULongToBin(unsigned int Val,unsigned char *Buf);	
	unsigned short BinToUShort(unsigned char *Buf);	
	unsigned int BinToULong(unsigned char *Buf);	

	// Pruefen ob min. ein Datensatz im AEIII+ vorhanden.
	int CheckData(unsigned char deviceIDRemote, int *piError);
	// Lesen eines einzelnen Datensatzes. Online zuerst.
	int ReadRecord(unsigned char deviceIDRemote, unsigned char *pucRxBuf, int *piRxCount, int *piError);
	// Quittiert den zuletzt gelesenen Datensatz um naechsten anzufordern.
	int QuitRecord(unsigned char deviceIDRemote, unsigned char *pucRxBuf, int *piRxCount, int *piError);
	// Loescht alle Datensaetzte im AEIII+
	int ClearData(unsigned char deviceIDRemote, int *piError);

	int SetGlobVar(unsigned char deviceIDRemote, unsigned char *pucVar, int iType, char *value, int *piError);
	int GetGlobVar(unsigned char deviceIDRemote, unsigned char *pucVar, int iType, char *value, int iValueLength, int *piError);
	// Relais setzten/ruecksetzen und zustand abfragen.
	int CloseRelay(unsigned char deviceIDRemote, int iNum, int iTime, int *piError);
	int GetRelayState(unsigned char deviceIDRemote, int iNum, int *piState, int *piTime, int *piError);
	int OpenRelay(unsigned char deviceIDRemote, int iNum, int *piError);

	// Einfache Anfrage ob Geraet mit Protokollversion 1 erreichbar.
	int Request(unsigned char deviceIDRemote);
	int Request2(unsigned char deviceIDRemote);

	// Ermitteln der Hardwareinfo zum Geraet
	int GetHardwareInfo(short deviceIDRemote, TDFHardwareInfo &hwInfo, int *piError);

protected:
	unsigned char m_pucRxBuf[MAX_MB_PAKET]; // MOD-Bus Read-Buffer
	unsigned char m_pucTxBuf[MAX_MB_PAKET]; // MOD-Bus Write-Buffer
	unsigned char m_transmit[MAX_MB_PAKET + 21];
	unsigned char m_receive[MAX_MB_PAKET + 21];

	// Sende und Empfang von Modbus - Kommando
	virtual int Command(unsigned char *pucTxBuf, int iTxCount, 
				unsigned char *pucRxBuf, int *piRxCount, 
				int *piError);
	int NewMB(unsigned short deviceIDRemote,unsigned char *buf, int *Len, int *Err);
	int GetModus(int *error);
	int Command2(unsigned char *sbuf,int cnt,unsigned char *ebuf,int &ret, int *Err);

	int WriteXByte (unsigned int Add, int Cnt, unsigned char *Buf, int *piError);

public:
	// Schreiben an Geraetespeicheradresse
	virtual int WriteMem(unsigned int Add, int Cnt, unsigned char *Buf, int *piError, int MaxB = MAX_MB_PAKET_DATA);
	// Lesen von Geraetespeicheradresse
	virtual int ReadMem(unsigned int Add, int Cnt, unsigned char *Buf, int *piError);
	int ReadXByte (unsigned int Add, int Cnt, unsigned char *Buf, int *piError);

protected:
	// CRC16 - Funktionen
//	void MkMBPaket(unsigned char *buf, int cnt);
//	int CheckMBPaket(unsigned char *buf, int cnt);
	void MkPaketCrc(unsigned char *buf,int cnt);
	int CheckPaketCrc(unsigned char *buf, int cnt);

	int (*m_callBack1)(void);


public:
	void SetChannelBase(CChannelBase& channelBase) { m_channelBase = &channelBase; }
	CChannelBase& GetChannelBase() { return (CChannelBase&)*m_channelBase; }

	unsigned char GetDeviceID() { return m_deviceID; }
	unsigned char SetDeviceID(unsigned char deviceID) { 

		unsigned char deviceIDPrev = m_deviceID; 
		m_deviceID = deviceID; 
		return deviceIDPrev; 
	}
	unsigned char GetDeviceIDRemote() { return m_deviceIDRemote; }
	unsigned char SetDeviceIDRemote(unsigned char deviceIDRemote) { 

		unsigned char deviceIDPrev = m_deviceIDRemote; 
		m_deviceIDRemote = deviceIDRemote; 
		return deviceIDPrev; 
	}

private:
	unsigned char m_protocolVersion; // 0 = Unbestimmt, 1 = AEIII+, 2 = PZE - Master IV
	CChannelBase *m_channelBase;
	unsigned char m_deviceID; // Die zu verwendende DeviceID der Klasse
	unsigned char m_deviceIDRemote; // Die zu verwendete DeviceID des Remote der Klasse
};

#endif // !defined(AFX_AE3MODBUS_H__783E8448_0B59_4DCC_B429_BDCAA018A25B__INCLUDED_)

