/**
 * @file  ATmega.cpp
 * @brief Implementierung der Klasse CATmega.
 */
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "ATmega.h"
#include <memory.h>
#ifndef _WIN32
#include <string.h>
#endif


CATmega::CATmega()
{
	// Speicher reservieren
	m_eeprom = new unsigned char[ATMEGA_SIZE_EEPROM];
	m_program = new unsigned char[ATMEGA_SIZE_PROGRAM];
	m_eepromUse = new unsigned int[ATMEGA_SIZE_EEPROM / 32];
	m_programUse = new unsigned int[ATMEGA_SIZE_PROGRAM / 32];

	// Initialisierung
	Reset();
}

CATmega::~CATmega()
{
	// Speicher freigeben
	if (m_eeprom) delete[] m_eeprom;
	m_eeprom = NULL;
	if (m_program) delete[] m_program;
	m_program = NULL;
	if (m_eepromUse) delete[] m_eepromUse;
	m_eepromUse = NULL;
	if (m_programUse) delete[] m_programUse;
	m_programUse = NULL;
}

void CATmega::Reset()
{
	// Speicher initialisieren
	memset(m_eeprom, 0xFF, ATMEGA_SIZE_EEPROM);
	memset(m_program, 0xFF, ATMEGA_SIZE_PROGRAM);
	memset(m_eepromUse, 0, sizeof(unsigned int)*(ATMEGA_SIZE_EEPROM / 32));
	memset(m_programUse, 0,  sizeof(unsigned int)*(ATMEGA_SIZE_PROGRAM / 32));
}

void CATmega::SetEepromUsebit(unsigned int address)
{
	m_eepromUse[address / 32] |= (1 << (address % 32));
}

void CATmega::SetProgramUsebit(unsigned int address)
{
	m_programUse[address / 32] |= (1 << (address % 32));
}

unsigned char CATmega::GetEepromUsebit(unsigned int address)
{
	return (m_eepromUse[address / 32] & (1 << (address % 32))) ? 1 : 0;
}

unsigned char CATmega::GetProgramUsebit(unsigned int address)
{
	return (m_programUse[address / 32] & (1 << (address % 32))) ? 1 : 0;
}

unsigned int CATmega::GetProgramFirstAddress()
{
	unsigned int address, firstAddress = 0;
	for (address=0; address < ATMEGA_SIZE_PROGRAM; address++) {

		if (GetProgramUsebit(address)) {

			firstAddress = address;
			break;
		}
	}

	return firstAddress;
}

unsigned int CATmega::GetProgramLastAddress()
{
	unsigned int address, lastAddress = 0;
	for (address=0; address < ATMEGA_SIZE_PROGRAM; address++) {

		if (GetProgramUsebit(address)) {

			lastAddress = address;
		}
	}

	return lastAddress;
}

unsigned int CATmega::GetEepromFirstAddress()
{
	unsigned int address, firstAddress = 0;
	for (address=0; address < ATMEGA_SIZE_EEPROM; address++) {

		if (GetEepromUsebit(address)) {

			firstAddress = address;
			break;
		}
	}

	return firstAddress;
}

unsigned int CATmega::GetEepromLastAddress()
{
	unsigned int address, lastAddress = 0;
	for (address=0; address < ATMEGA_SIZE_EEPROM; address++) {

		if (GetEepromUsebit(address)) {

			lastAddress = address;
		}
	}

	return lastAddress;
}

unsigned int CATmega::GetProgramSize()
{
	return GetProgramLastAddress() - GetProgramFirstAddress() + 1;
}

