/**
 * @file  DFFileDisplayColors.cpp
 * @brief Implementierung der Klasse CDFFileDisplayColors.
 */
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "DFFileDisplayColors.h"
#include <memory.h>
#include <stdio.h>

CDFFileDisplayColors::CDFFileDisplayColors()
{
	// CRC32 Initialisierung
	InitCRC32Table();

	// Ausgangseinstellung
	ResetError();
	Reset();
}

CDFFileDisplayColors::~CDFFileDisplayColors()
{
}

void CDFFileDisplayColors::Reset()
{
	// Basis Reset
	CDFFileIntelHex::Reset();

	// Member initialisieren
	
	// Groessen setzen
	m_sizeData   = 0;

	// Speicher fuer DisplayColors loeschen
	memset(m_memDisplayColors, 0x00, MAX_MEM_DISPLAYCOLORS);

	// Hilfsstrukturen aufsetzen
	m_pDisplayColorsNode   = (DISPLAYCOLORNODE*)m_memDisplayColors;
	m_pDisplayColorsNode->version = 1;
	m_pDisplayColorsNode->cnt     = 0;
	m_pDisplayColorsNode->temp[0] = 0;
	m_pDisplayColorsNode->temp[1] = 0;
	m_pDisplayColorsNode->ptrColorArray = sizeof(DISPLAYCOLORNODE);
}

void CDFFileDisplayColors::Init( unsigned char count )
{
	int i, j;

	// Speicher fuer TouchKeyDaten loeschen
	memset(m_memDisplayColors, 0x00, MAX_MEM_DISPLAYCOLORS);

	// 1.
	// Zeichensaetze init
	// Grundstruktur aufsetzen
	m_pDisplayColorsNode->version = 1;
	m_pDisplayColorsNode->cnt     = count;
	m_pDisplayColorsNode->temp[0] = 0;
	m_pDisplayColorsNode->temp[1] = 0;
	m_pDisplayColorsNode->ptrColorArray = sizeof(DISPLAYCOLORNODE);

	// Zeigerarray und DisplayColorsstrukturzeiger initialisieren
	i = m_pDisplayColorsNode->cnt;
	for (j=0; j < i; j++) {
		
		// Zeigereintrag auf DisplayColorstruktur
		*(unsigned int*)&m_memDisplayColors[m_pDisplayColorsNode->ptrColorArray + j * sizeof(unsigned int)] = 
			m_pDisplayColorsNode->ptrColorArray + i * sizeof(unsigned int) + j * sizeof(DISPLAYCOLOR);

		m_pDisplayColors[j] = (DISPLAYCOLOR*)&m_memDisplayColors[m_pDisplayColorsNode->ptrColorArray + i * sizeof(unsigned int) + j * sizeof(DISPLAYCOLOR)];
	}

	// Variable fuer Groessenangabe
	m_sizeData = sizeof(DISPLAYCOLORNODE) + m_pDisplayColorsNode->cnt * sizeof(unsigned int) + m_pDisplayColorsNode->cnt * sizeof(DISPLAYCOLOR);
}

int CDFFileDisplayColors::SetDisplayColor(int idx, DISPLAYCOLOR dc)
{
	if (idx >= m_pDisplayColorsNode->cnt)
	{
		return -1;
	}

	*m_pDisplayColors[idx] = dc;
	return idx;
}

DISPLAYCOLOR CDFFileDisplayColors::GetDisplayColor(int idx)
{
	DISPLAYCOLOR dc;
	memset(&dc, 0, sizeof(DISPLAYCOLOR));

	if (idx >= m_pDisplayColorsNode->cnt)
	{
		return dc;
	}

	return *m_pDisplayColors[idx];
}

int CDFFileDisplayColors::Load(char *fileName) 
{
	// Lesen der Hexdatei
	FILE *fp;
	char data[0x400];

	// Fehlerobjekt zuruecksetzen
	ResetError();

	// Umgebung zuruecksetzen
	Reset();

	if ((fp = fopen(fileName, "r")) == (FILE*)0) {
		// Oeffnen der Datei gescheitert
		m_errorState = ES_FILE_OPEN;
		sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_FILE_OPEN_TEXT);
		return 0;
	}

	// Daten eintragen
	unsigned char type;
	unsigned short address;

	while(!feof(fp) && (m_errorState == ES_NO)) {
	
		if (fgets(data, 0x400, fp) != 0) {
			
			address = GetAddress(data);
			type    = GetType(data);

			switch (type) {

			case INTELHEX_TYPE_DISPLAYCOLOR_HEADER:
				break;

			case INTELHEX_TYPE_DISPLAYCOLOR_DATA:
				m_sizeData += GetData(data, &m_memDisplayColors[address]);
				break;

			case INTELHEX_TYPE_EOF:
				break;
			}
		}
	}
	fclose(fp);	

	// Member durch gelesene Daten initialisieren und pruefen
	if (m_sizeData) {
		
		int i, j;

		// 1.
		// DisplayColors pruefen

		// Zeigerarray und DisplayColorstrukturzeiger initialisieren
		i = m_pDisplayColorsNode->cnt;
		if (i > MAX_DISPLAYCOLORS) {

			m_errorState = ES_FORMAT;
			sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_FORMAT_TEXT);
			return 0;
		}

		for (j=0; j < i; j++) {
			
			m_pDisplayColors[j] = (DISPLAYCOLOR*)&m_memDisplayColors[m_pDisplayColorsNode->ptrColorArray + i * sizeof(unsigned int) + j * sizeof(DISPLAYCOLOR)];
		}

	} else {

		m_errorState = ES_NO_DATA;
		sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_NO_DATA_TEXT);
		return 0;
	}

	return 1;
}



int CDFFileDisplayColors::Save(char *fileName, unsigned char append) 
{
	FILE *fp;
	char szData[0x400];

	// Fehlerobjekt zuruecksetzen
	ResetError();

	// vorhandene Datei anlegen
	if (!append) {
		if ((fp = fopen(fileName, "w")) == (FILE*)0) {
			// Oeffnen der Datei gescheitert
			m_errorState = ES_FILE_OPEN;
			sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_FILE_OPEN_TEXT);
			return 0;
		}
		fclose(fp);
	}

	// Daten der DisplayColors sichern
	if (!SaveDisplayColors(fileName, 1)) {

		return 0;
	}

	if (!append) {
		// Endezeile in Datei eintragen
		if ((fp = fopen(fileName, "a")) == (FILE*)0) {
			// Oeffnen der Datei gescheitert
			m_errorState = ES_FILE_OPEN;
			sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_FILE_OPEN_TEXT);
			return 0;
		}

		if (!MakeEndLine(szData)) {
			
			fclose(fp);
			m_errorState = ES_INTEL_HEX;
			sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_INTEL_HEX_TEXT);
			return 0;
		}

		fprintf(fp, "%s\n", szData);
		fclose(fp);
	}

	return 1;
}

int CDFFileDisplayColors::SaveDisplayColors(char *fileName, unsigned char append) 
{
	FILE *fp;
	char szData[0x400];

	if ((fp = fopen(fileName, (append) ? "a" : "w")) == (FILE*)0) {
		// Oeffnen der Datei gescheitert
		m_errorState = ES_FILE_OPEN;
		sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_FILE_OPEN_TEXT);
		return 0;
	}

	// Daten der DisplayColors eintragen
	unsigned int ulPos;
	for (ulPos=0; ulPos < m_sizeData; ulPos += INTELHEX_DATA_COUNT) {

		if (!MakeTypedDataLine((unsigned short)ulPos, INTELHEX_TYPE_DISPLAYCOLOR_DATA, 
			&m_memDisplayColors[ulPos], (unsigned char)(((ulPos + INTELHEX_DATA_COUNT) < m_sizeData) ? INTELHEX_DATA_COUNT : m_sizeData - ulPos), szData)) {
			
			m_errorState = ES_INTEL_HEX;
			sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_INTEL_HEX_TEXT);
			break;
		}

		fprintf(fp, "%s\n", szData);
	}

	// Abbruch nach Fehler
	if (m_errorState != ES_NO) {

		fclose(fp);
		return 0;
	}

	if (!append) {

		if (!MakeEndLine(szData)) {
			
			fclose(fp);
			m_errorState = ES_INTEL_HEX;
			sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_INTEL_HEX_TEXT);
			return 0;
		}

		fprintf(fp, "%s\n", szData);
	}

	fclose(fp);

	return 1;
}

