/**
 * @file  DFFileTouchKeys.cpp
 * @brief Implementierung der Klasse CDFFileTouchKeys.
 */
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "DFFileTouchKeys.h"
#include <memory.h>
#include <stdio.h>

CDFFileTouchKeys::CDFFileTouchKeys()
{
	// CRC32 Initialisierung
	InitCRC32Table();

	// Ausgangseinstellung
	ResetError();
	Reset();
}

CDFFileTouchKeys::~CDFFileTouchKeys()
{
}

void CDFFileTouchKeys::Reset()
{
	// Basis Reset
	CDFFileIntelHex::Reset();

	// Member initialisieren
	
	// Groessen setzen
	m_sizeData   = 0;

	// Speicher fuer TouchKeys loeschen
	memset(m_memTouchKeys, 0x00, MAX_MEM_TOUCHKEYS);

	// Hilfsstrukturen aufsetzen
	m_pTouchKeysNode   = (TOUCHKEYNODE*)m_memTouchKeys;
	m_pTouchKeysNode->version = 1;
	m_pTouchKeysNode->cnt     = 0;
	m_pTouchKeysNode->temp[0] = 0;
	m_pTouchKeysNode->temp[1] = 0;
	m_pTouchKeysNode->ptrTouchkeyArray = sizeof(TOUCHKEYNODE);
}

void CDFFileTouchKeys::Init(unsigned char count)
{
	int i, j;

	// Speicher fuer TouchKeyDaten loeschen
	memset(m_memTouchKeys, 0x00, MAX_MEM_TOUCHKEYS);

	// 1.
	// Zeichensaetze init
	// Grundstruktur aufsetzen
	m_pTouchKeysNode->version = 1;
	m_pTouchKeysNode->cnt     = count;
	m_pTouchKeysNode->temp[0] = 0;
	m_pTouchKeysNode->temp[1] = 0;
	m_pTouchKeysNode->ptrTouchkeyArray = sizeof(TOUCHKEYNODE);

	// Zeigerarray und Touchkeysstrukturzeiger initialisieren
	i = m_pTouchKeysNode->cnt;
	for (j=0; j < i; j++) {
		
		// Zeigereintrag auf Touchkeystruktur
		*(unsigned int*)&m_memTouchKeys[m_pTouchKeysNode->ptrTouchkeyArray + j * sizeof(unsigned int)] = 
			m_pTouchKeysNode->ptrTouchkeyArray + i * sizeof(unsigned int) + j * sizeof(TOUCHKEY);

		m_pTouchKeys[j] = (TOUCHKEY*)&m_memTouchKeys[m_pTouchKeysNode->ptrTouchkeyArray + i * sizeof(unsigned int) + j * sizeof(TOUCHKEY)];
	}

	// Variable fuer Groessenangabe
	m_sizeData = sizeof(TOUCHKEYNODE) + m_pTouchKeysNode->cnt * sizeof(unsigned int) + m_pTouchKeysNode->cnt * sizeof(TOUCHKEY);
}

int CDFFileTouchKeys::SetTouchKey(int idx, TOUCHKEY tk)
{
	if (idx >= m_pTouchKeysNode->cnt)
	{
		return -1;
	}

	*m_pTouchKeys[idx] = tk;
	return idx;
}

TOUCHKEY CDFFileTouchKeys::GetTouchKey(int idx)
{
	TOUCHKEY tk;
	memset(&tk, 0, sizeof(TOUCHKEY));

	if (idx >= m_pTouchKeysNode->cnt)
	{
		return tk;
	}

	return *m_pTouchKeys[idx];
}

int CDFFileTouchKeys::Load(char *fileName) 
{
	// Lesen der Hexdatei
	FILE *fp;
	char data[0x400];

	// Fehlerobjekt zuruecksetzen
	ResetError();

	// Umgebung zuruecksetzen
	Reset();

	if ((fp = fopen(fileName, "r")) == (FILE*)0) {
		// Oeffnen der Datei gescheitert
		m_errorState = ES_FILE_OPEN;
		sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_FILE_OPEN_TEXT);
		return 0;
	}

	// Daten eintragen
	unsigned char type;
	unsigned short address;

	while(!feof(fp) && (m_errorState == ES_NO)) {
	
		if (fgets(data, 0x400, fp) != 0) {
			
			address = GetAddress(data);
			type    = GetType(data);

			switch (type) {

			case INTELHEX_TYPE_TOUCHKEY_HEADER:
				break;

			case INTELHEX_TYPE_TOUCHKEY_DATA:
				m_sizeData += GetData(data, &m_memTouchKeys[address]);
				break;

			case INTELHEX_TYPE_EOF:
				break;
			}
		}
	}
	fclose(fp);	

	// Member durch gelesene Daten initialisieren und pruefen
	if (m_sizeData) {
		
		int i, j;

		// 1.
		// Touchkeys pruefen

		// Zeigerarray und Touchkeystrukturzeiger initialisieren
		i = m_pTouchKeysNode->cnt;
		if (i > MAX_TOUCHKEYS) {

			m_errorState = ES_FORMAT;
			sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_FORMAT_TEXT);
			return 0;
		}

		for (j=0; j < i; j++) {
			
			m_pTouchKeys[j] = (TOUCHKEY*)&m_memTouchKeys[m_pTouchKeysNode->ptrTouchkeyArray + i * sizeof(unsigned int) + j * sizeof(TOUCHKEY)];
		}

	} else {

		m_errorState = ES_NO_DATA;
		sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_NO_DATA_TEXT);
		return 0;
	}

	return 1;
}



int CDFFileTouchKeys::Save(char *fileName, unsigned char append) 
{
	FILE *fp;
	char szData[0x400];

	// Fehlerobjekt zuruecksetzen
	ResetError();

	// vorhandene Datei anlegen
	if (!append) {
		if ((fp = fopen(fileName, "w")) == (FILE*)0) {
			// Oeffnen der Datei gescheitert
			m_errorState = ES_FILE_OPEN;
			sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_FILE_OPEN_TEXT);
			return 0;
		}
		fclose(fp);
	}

	// Daten der TouchKeys sichern
	if (!SaveTouchKeys(fileName, 1)) {

		return 0;
	}

	if (!append) {
		// Endezeile in Datei eintragen
		if ((fp = fopen(fileName, "a")) == (FILE*)0) {
			// Oeffnen der Datei gescheitert
			m_errorState = ES_FILE_OPEN;
			sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_FILE_OPEN_TEXT);
			return 0;
		}

		if (!MakeEndLine(szData)) {
			
			fclose(fp);
			m_errorState = ES_INTEL_HEX;
			sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_INTEL_HEX_TEXT);
			return 0;
		}

		fprintf(fp, "%s\n", szData);
		fclose(fp);
	}

	return 1;
}

int CDFFileTouchKeys::SaveTouchKeys(char *fileName, unsigned char append) 
{
	FILE *fp;
	char szData[0x400];

	if ((fp = fopen(fileName, (append) ? "a" : "w")) == (FILE*)0) {
		// Oeffnen der Datei gescheitert
		m_errorState = ES_FILE_OPEN;
		sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_FILE_OPEN_TEXT);
		return 0;
	}

	// Daten der TouchKeys eintragen
	unsigned int ulPos;
	for (ulPos=0; ulPos < m_sizeData; ulPos += INTELHEX_DATA_COUNT) {

		if (!MakeTypedDataLine((unsigned short)ulPos, INTELHEX_TYPE_TOUCHKEY_DATA, 
			&m_memTouchKeys[ulPos], (unsigned char)(((ulPos + INTELHEX_DATA_COUNT) < m_sizeData) ? INTELHEX_DATA_COUNT : m_sizeData - ulPos), szData)) {
			
			m_errorState = ES_INTEL_HEX;
			sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_INTEL_HEX_TEXT);
			break;
		}

		fprintf(fp, "%s\n", szData);
	}

	// Abbruch nach Fehler
	if (m_errorState != ES_NO) {

		fclose(fp);
		return 0;
	}

	if (!append) {

		if (!MakeEndLine(szData)) {
			
			fclose(fp);
			m_errorState = ES_INTEL_HEX;
			sprintf(m_errorDetail, "%s(%i): %s", __FILE__, __LINE__, ES_INTEL_HEX_TEXT);
			return 0;
		}

		fprintf(fp, "%s\n", szData);
	}

	fclose(fp);

	return 1;
}

